

#include <Arduino.h>
#include "skp_lvgl.h"
#include <lvgl.h>
#include <Wire.h>
#include <ILI9488_t3.h>
#include <SPI.h>
#include <Adafruit_FT6206.h>
#include "ui.h"
#include "ui_helpers.h"

int LCD_BL = 14;
int LCD_RST = 15;

#define TFT_CS 10
#define TFT_DC  9

ILI9488_t3 display = ILI9488_t3(&SPI, TFT_CS, TFT_DC);
Adafruit_FT6206 ts = Adafruit_FT6206();

int oldTouchX = 0;
int oldTouchY = 0;

static const uint16_t screenWidth  = 480;
static const uint16_t screenHeight = 320;
static lv_disp_draw_buf_t draw_buf;
static lv_color_t buf[ screenWidth * 10 ];

/* Display flushing */
void my_disp_flush(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p)
{ 
   uint16_t width = (area->x2 - area->x1 +1);
   uint16_t height = (area->y2 - area->y1+1);

   display.writeRect(area->x1, area->y1, width, height, (uint16_t *)color_p);
   lv_disp_flush_ready(disp); /* tell lvgl that flushing is done */
}

bool my_touchpad_read(lv_indev_t * indev, lv_indev_data_t * data)
{
    uint16_t touchX, touchY;
  
    if (ts.touched())
    {   
        // Retrieve a point  
        TS_Point p = ts.getPoint(); 
    
        touchX = p.y;         // Rotate the co-ordinates
        touchY = p.x;
        touchY = 320-touchY;
  
         if ((touchX != oldTouchX) || (touchY != oldTouchY))
         {
              Serial.print("x= ");
              Serial.print(touchX,DEC);
              Serial.print(" y= ");
              Serial.println(touchY,DEC);
              
              oldTouchY = touchY;
              oldTouchX = touchX;
              data->state = LV_INDEV_STATE_PR; 
              data->point.x = touchX;
              data->point.y = touchY;
         
         }
    }else
    {
        data->point.x = oldTouchX;
        data->point.y = oldTouchY;
        data->state =LV_INDEV_STATE_REL;
     }
           
    return 0;
}

void skp_lvgl_init(void)
{
    pinMode(LCD_BL,OUTPUT);
    digitalWrite(LCD_BL, HIGH); 
    lv_init();
    display.begin();
    display.fillScreen(ILI9488_BLUE);
    delay(400);
    display.setRotation(1); 

    lv_disp_draw_buf_init( &draw_buf, buf, NULL, screenWidth * 10 );

    /*Initialize the display*/
    static lv_disp_drv_t disp_drv;
    lv_disp_drv_init( &disp_drv );
    /*Change the following line to your display resolution*/
    disp_drv.hor_res = screenWidth;
    disp_drv.ver_res = screenHeight;
    disp_drv.flush_cb = my_disp_flush;
    disp_drv.draw_buf = &draw_buf;
    lv_disp_drv_register( &disp_drv );

    /*Initialize the (dummy) input device driver*/
    static lv_indev_drv_t indev_drv;
    lv_indev_drv_init( &indev_drv );
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = my_touchpad_read;
    lv_indev_drv_register( &indev_drv );
 //   lv_disp_buf_init(&disp_buf, buf, NULL, LV_HOR_RES_MAX * LV_VER_RES_MAX / 10);
   
    ui_init();
  
}
